# @summary Manage kernel
#
# @example
#   include ::kernel
#
# @param version
#   The desired kernel version to be installed.
#   Must be the full string that would be output by running `uname -r`.
#   Default of 'UNSET' disables management of kernel package version.
# @param package_ensure
#   Package ensure parameter.
# @param package_name
#   The kernel package name.
# @param install_devel
#   Boolean to specify whether the kernel-devel package should be installed.
# @param devel_package_name
#   The kernel development package name.
# @param install_headers
#   Boolean to specify whether the kernel-headers package should be installed.
# @param headers_package_name
#   The kernel headers package name.
# @param install_firmware
#   Boolean to specify whether the kernel-firmware package should be installed.
# @param firmware_package_name
#   The kernel firmware package name.
# @param set_default_kernel
#   Boolean to specify whether if the default kernel in GRUB should be modified by this module.
# @param grub_default_kernel
#   The kernel version to set as default in GRUB.
#   Default is the value of `$version`.
# @param grub_conf_path
#   The path to GRUB configuration file.  Default is OS dependent.
# @param grub_class
#   The class to be used to manage GRUB.
#
class kernel (
  Optional[String[1]] $version = undef,
  Enum['present', 'latest'] $package_ensure   = 'present',
  String[1] $package_name = 'kernel',
  Boolean $install_devel = false,
  String[1] $devel_package_name = 'kernel-devel',
  Boolean $install_headers = false,
  String[1] $headers_package_name = 'kernel-headers',
  Boolean $install_firmware = false,
  String[1] $firmware_package_name = 'kernel-firmware',
  Boolean $set_default_kernel = true,
  Optional[String[1]] $grub_default_kernel = undef,
  Stdlib::Unixpath $grub_conf_path = '/etc/default/grub',
  Enum['grubby'] $grub_class = 'grubby',
) {
  if $version {
    if $version =~ /\.${facts['os']['architecture']}/ {
      $kernel_version = $version
    } else {
      $kernel_version = "${version}.${facts['os']['architecture']}"
    }
  } else {
    $kernel_version = undef
  }

  contain kernel::install
  contain kernel::config
  contain "kernel::${grub_class}"

  Class['kernel::install']
  -> Class['kernel::config']
  -> Class["kernel::${grub_class}"]
}
